@@ -26,6 +26,7 @@
 #include <gobject/gvaluecollector.h>
 
 #include "egg-list-box.h"
+#include "egg-list-box-accessible.h"
 
 /* This already exists in gtk as _gtk_marshal_VOID__ENUM_INT, inline it here for now
    to avoid separate marshallers file */
@@ -63,7 +64,7 @@ struct _EggListBoxPrivate
   GHashTable *child_hash;
   GHashTable *separator_hash;
 
-  GCompareDataFunc sort_func;
+  EggListBoxSortFunc sort_func;
   gpointer sort_func_target;
   GDestroyNotify sort_func_target_destroy_notify;
 
@@ -112,7 +113,10 @@ enum {
 };
 
 enum  {
-  PROP_0
+  PROP_0,
+  PROP_SELECTION_MODE,
+  PROP_ACTIVATE_ON_SINGLE_CLICK,
+  LAST_PROPERTY
 };
 
 G_DEFINE_TYPE (EggListBox, egg_list_box, GTK_TYPE_CONTAINER)
@@ -210,6 +214,7 @@ static void                 egg_list_box_real_get_preferred_width_for_height (Gt
 									      gint                *minimum_width,
 									      gint                *natural_width);
 
+static GParamSpec *properties[LAST_PROPERTY] = { NULL, };
 static guint signals[LAST_SIGNAL] = { 0 };
 
 static EggListBoxChildInfo*
@@ -255,6 +260,50 @@ egg_list_box_init (EggListBox *list_box)
   priv->separator_hash = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, NULL);
 }
 
+static void
+egg_list_box_get_property (GObject    *obj,
+                           guint       property_id,
+                           GValue     *value,
+                           GParamSpec *pspec)
+{
+  EggListBox *list_box = EGG_LIST_BOX (obj);
+
+  switch (property_id)
+    {
+    case PROP_SELECTION_MODE:
+      g_value_set_enum (value, list_box->priv->selection_mode);
+      break;
+    case PROP_ACTIVATE_ON_SINGLE_CLICK:
+      g_value_set_boolean (value, list_box->priv->activate_single_click);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, property_id, pspec);
+      break;
+    }
+}
+
+static void
+egg_list_box_set_property (GObject      *obj,
+                           guint         property_id,
+                           const GValue *value,
+                           GParamSpec   *pspec)
+{
+  EggListBox *list_box = EGG_LIST_BOX (obj);
+
+  switch (property_id)
+    {
+    case PROP_SELECTION_MODE:
+      egg_list_box_set_selection_mode (list_box, g_value_get_enum (value));
+      break;
+    case PROP_ACTIVATE_ON_SINGLE_CLICK:
+      egg_list_box_set_activate_on_single_click (list_box, g_value_get_boolean (value));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, property_id, pspec);
+      break;
+    }
+}
+
 static void
 egg_list_box_finalize (GObject *obj)
 {
@@ -293,6 +342,10 @@ egg_list_box_class_init (EggListBoxClass *klass)
 
   g_type_class_add_private (klass, sizeof (EggListBoxPrivate));
 
+  gtk_widget_class_set_accessible_type (widget_class, EGG_TYPE_LIST_BOX_ACCESSIBLE);
+
+  object_class->get_property = egg_list_box_get_property;
+  object_class->set_property = egg_list_box_set_property;
   object_class->finalize = egg_list_box_finalize;
   widget_class->enter_notify_event = egg_list_box_real_enter_notify_event;
   widget_class->leave_notify_event = egg_list_box_real_leave_notify_event;
@@ -321,6 +374,23 @@ egg_list_box_class_init (EggListBoxClass *klass)
   klass->move_cursor = egg_list_box_real_move_cursor;
   klass->refilter = egg_list_box_real_refilter;
 
+  properties[PROP_SELECTION_MODE] =
+    g_param_spec_enum ("selection-mode",
+                       "Selection mode",
+                       "The selection mode",
+                       GTK_TYPE_SELECTION_MODE,
+                       GTK_SELECTION_SINGLE,
+                       G_PARAM_READWRITE);
+
+  properties[PROP_ACTIVATE_ON_SINGLE_CLICK] =
+    g_param_spec_boolean ("activate-on-single-click",
+                          "Activate on Single Click",
+                          "Activate row on a single click",
+                          TRUE,
+                          G_PARAM_READWRITE);
+
+  g_object_class_install_properties (object_class, LAST_PROPERTY, properties);
+
   signals[CHILD_SELECTED] =
     g_signal_new ("child-selected",
 		  EGG_TYPE_LIST_BOX,
@@ -404,6 +474,14 @@ egg_list_box_class_init (EggListBoxClass *klass)
 				"toggle-cursor-child", 0, NULL);
 }
 
+/**
+ * egg_list_box_get_selected_child:
+ * @self: An #EggListBox.
+ *
+ * Gets the selected child.
+ *
+ * Return value: (transfer none): The selected #GtkWidget.
+ **/
 GtkWidget *
 egg_list_box_get_selected_child (EggListBox *list_box)
 {
@@ -417,6 +495,15 @@ egg_list_box_get_selected_child (EggListBox *list_box)
   return NULL;
 }
 
+/**
+ * egg_list_box_get_child_at_y:
+ * @self: An #EggListBox.
+ * @y: position
+ *
+ * Gets the child at the position.
+ *
+ * Return value: (transfer none): The child #GtkWidget.
+ **/
 GtkWidget *
 egg_list_box_get_child_at_y (EggListBox *list_box, gint y)
 {
@@ -432,6 +519,11 @@ egg_list_box_get_child_at_y (EggListBox *list_box, gint y)
 }
 
 
+/**
+ * egg_list_box_select_child:
+ * @self: a #EggListBox
+ * @child: (allow-none): The child to select or %NULL
+ */
 void
 egg_list_box_select_child (EggListBox *list_box, GtkWidget *child)
 {
@@ -468,8 +560,7 @@ egg_list_box_add_to_scrolled (EggListBox *list_box,
   g_return_if_fail (list_box != NULL);
   g_return_if_fail (scrolled != NULL);
 
-  gtk_scrolled_window_add_with_viewport (scrolled,
-					 GTK_WIDGET (list_box));
+  gtk_container_add (GTK_CONTAINER (scrolled), GTK_WIDGET (list_box));
   egg_list_box_set_adjustment (list_box,
 			       gtk_scrolled_window_get_vadjustment (scrolled));
 }
@@ -484,13 +575,18 @@ egg_list_box_set_selection_mode (EggListBox *list_box, GtkSelectionMode mode)
 
   if (mode == GTK_SELECTION_MULTIPLE)
     {
-      g_warning ("egg-list-box.vala:115: Multiple selections not supported");
+      g_warning ("Multiple selections not supported");
       return;
     }
 
+  if (priv->selection_mode == mode)
+    return;
+
   priv->selection_mode = mode;
   if (mode == GTK_SELECTION_NONE)
     egg_list_box_update_selected (list_box, NULL);
+
+  g_object_notify_by_pspec (G_OBJECT (list_box), properties[PROP_SELECTION_MODE]);
 }
 
 
@@ -589,9 +685,15 @@ egg_list_box_reseparate (EggListBox *list_box)
   gtk_widget_queue_resize (GTK_WIDGET (list_box));
 }
 
+/**
+ * egg_list_box_set_sort_func:
+ * @f: (closure f_target):
+ * @f_target: (allow-none):
+ * @f_target_destroy_notify: (allow-none):
+ */
 void
 egg_list_box_set_sort_func (EggListBox *list_box,
-			    GCompareDataFunc f,
+			    EggListBoxSortFunc f,
 			    void *f_target,
 			    GDestroyNotify f_target_destroy_notify)
 {
@@ -648,7 +750,14 @@ egg_list_box_set_activate_on_single_click (EggListBox *list_box,
 
   g_return_if_fail (list_box != NULL);
 
+  single = single != FALSE;
+
+  if (priv->activate_single_click == single)
+    return;
+
   priv->activate_single_click = single;
+
+  g_object_notify_by_pspec (G_OBJECT (list_box), properties[PROP_ACTIVATE_ON_SINGLE_CLICK]);
 }
 
 static void
@@ -709,6 +818,7 @@ egg_list_box_update_cursor (EggListBox *list_box,
 				 priv->cursor_child->y + allocation.y,
 				 priv->cursor_child->y + allocation.y + priv->cursor_child->height);
   }
+  _egg_list_box_accessible_update_cursor (list_box, child ? child->widget : NULL);
 }
 
 static void
@@ -725,6 +835,7 @@ egg_list_box_update_selected (EggListBox *list_box,
 		     (priv->selected_child != NULL) ? priv->selected_child->widget : NULL);
       gtk_widget_queue_draw (GTK_WIDGET (list_box));
     }
+  _egg_list_box_accessible_selection_changed (list_box);
   if (child != NULL)
     egg_list_box_update_cursor (list_box, child);
 }
@@ -844,7 +955,7 @@ egg_list_box_real_button_press_event (GtkWidget *widget,
   EggListBox *list_box = EGG_LIST_BOX (widget);
   EggListBoxPrivate *priv = list_box->priv;
 
-  if (event->button == 1)
+  if (event->button == GDK_BUTTON_PRIMARY)
     {
       EggListBoxChildInfo *child;
       child = egg_list_box_find_child_at_y (list_box, event->y);
@@ -874,7 +985,7 @@ egg_list_box_real_button_release_event (GtkWidget *widget,
   EggListBox *list_box = EGG_LIST_BOX (widget);
   EggListBoxPrivate *priv = list_box->priv;
 
-  if (event->button == 1)
+  if (event->button == GDK_BUTTON_PRIMARY)
     {
       if (priv->active_child != NULL &&
           priv->active_child_active)
@@ -1033,8 +1144,9 @@ egg_list_box_real_focus (GtkWidget* widget, GtkDirectionType direction)
 	modify_selection_pressed = TRUE;
     }
 
-  egg_list_box_update_cursor (list_box, next_focus_child);
-  if (!modify_selection_pressed)
+  if (modify_selection_pressed)
+    egg_list_box_update_cursor (list_box, next_focus_child);
+  else
     egg_list_box_update_selected (list_box, next_focus_child);
 
   return TRUE;
