@@ -38,7 +38,6 @@
 #include <gdesktop-enums.h>
 
 #include "cc-rr-labeler.h"
-#include <libupower-glib/upower.h>
 
 CC_PANEL_REGISTER (CcDisplayPanel, cc_display_panel)
 
@@ -98,9 +97,6 @@ struct _CcDisplayPanelPrivate
 
   /* These are used while we are waiting for the ApplyConfiguration method to be executed over D-bus */
   GDBusProxy *proxy;
-
-  UpClient *up_client;
-  gboolean lid_is_closed;
 };
 
 typedef struct
@@ -155,10 +151,6 @@ cc_display_panel_set_property (GObject      *object,
 static void
 cc_display_panel_dispose (GObject *object)
 {
-  CcDisplayPanelPrivate *priv = CC_DISPLAY_PANEL (object)->priv;
-
-  g_clear_object (&priv->up_client);
-
   G_OBJECT_CLASS (cc_display_panel_parent_class)->dispose (object);
 }
 
@@ -701,20 +693,7 @@ rebuild_on_off_radios (CcDisplayPanel *self)
 
   if (!gnome_rr_config_get_clone (self->priv->current_configuration) && self->priv->current_output)
     {
-      gboolean display_closed = FALSE;
-
-      if (self->priv->lid_is_closed)
-        {
-          GnomeRROutput *output;
-
-          output = gnome_rr_screen_get_output_by_name (self->priv->screen,
-                                                       gnome_rr_output_info_get_name (self->priv->current_output));
-
-          display_closed = gnome_rr_output_is_laptop (output);
-        }
-
-      if ((count_active_outputs (self) > 1 || !gnome_rr_output_info_is_active (self->priv->current_output))
-          && !display_closed)
+      if (count_active_outputs (self) > 1 || !gnome_rr_output_info_is_active (self->priv->current_output))
         sensitive = TRUE;
       else
         sensitive = FALSE;
@@ -1359,12 +1338,6 @@ list_edges_for_output (GnomeRROutputInfo *output, GArray *edges)
 
   gnome_rr_output_info_get_geometry (output, &x, &y, &w, &h);
 
-  if (!gnome_rr_output_info_is_active (output))
-    {
-      h = gnome_rr_output_info_get_preferred_height (output);
-      w = gnome_rr_output_info_get_preferred_width (output);
-    }
-
   apply_rotation_to_geometry (output, &w, &h);
 
   /* Top, Bottom, Left, Right */
@@ -1565,12 +1538,6 @@ get_output_rect (GnomeRROutputInfo *output, GdkRectangle *rect)
 {
   gnome_rr_output_info_get_geometry (output, &rect->x, &rect->y, &rect->width, &rect->height);
 
-  if (!gnome_rr_output_info_is_active (output))
-    {
-      rect->width = gnome_rr_output_info_get_preferred_width (output);
-      rect->height = gnome_rr_output_info_get_preferred_height (output);
-    }
-
   apply_rotation_to_geometry (output, &rect->width, &rect->height);
 }
 
@@ -1774,13 +1741,6 @@ set_primary_output (CcDisplayPanel *self,
     gnome_rr_output_info_set_primary (outputs[i], outputs[i] == output);
 }
 
-static void
-grab_weak_ref_notify (gpointer  area,
-                      GObject  *object)
-{
-  foo_scroll_area_end_grab (area, NULL);
-}
-
 static void
 on_output_event (FooScrollArea *area,
                  FooScrollAreaEvent *event,
@@ -1823,7 +1783,6 @@ on_output_event (FooScrollArea *area,
 	  gnome_rr_output_info_get_geometry (output, &output_x, &output_y, NULL, NULL);
 
 	  foo_scroll_area_begin_grab (area, on_output_event, data);
-	  g_object_weak_ref (data, grab_weak_ref_notify, area);
 
 	  info = g_new0 (GrabInfo, 1);
 	  info->grab_x = event->x;
@@ -1862,7 +1821,7 @@ on_output_event (FooScrollArea *area,
 
 	  g_array_sort (snaps, compare_snaps);
 
-	  gnome_rr_output_info_set_geometry (output, old_x, old_y, width, height);
+	  gnome_rr_output_info_set_geometry (output, new_x, new_y, width, height);
 
 	  for (i = 0; i < snaps->len; ++i)
 	    {
@@ -1896,7 +1855,6 @@ on_output_event (FooScrollArea *area,
 
 	      g_free (g_object_get_data (G_OBJECT (output), "grab-info"));
 	      g_object_set_data (G_OBJECT (output), "grab-info", NULL);
-	      g_object_weak_unref (data, grab_weak_ref_notify, area);
 
 #if 0
               g_debug ("new position: %d %d %d %d", output->x, output->y, output->width, output->height);
@@ -2620,23 +2578,6 @@ cc_display_panel_init (CcDisplayPanel *self)
   g_resources_register (cc_display_get_resource ());
 }
 
-static void
-cc_display_panel_up_client_changed (UpClient       *client,
-                                    CcDisplayPanel *self)
-{
-  CcDisplayPanelPrivate *priv = self->priv;
-  gboolean lid_is_closed;
-
-  lid_is_closed = up_client_get_lid_is_closed (client);
-
-  if (lid_is_closed != priv->lid_is_closed)
-    {
-      priv->lid_is_closed = lid_is_closed;
-
-      rebuild_on_off_radios (self);
-    }
-}
-
 static GObject *
 cc_display_panel_constructor (GType                  gtype,
                               guint                  n_properties,
@@ -2741,21 +2682,5 @@ cc_display_panel_constructor (GType                  gtype,
   gtk_widget_show (self->priv->panel);
   gtk_container_add (GTK_CONTAINER (self), self->priv->panel);
 
-  self->priv->up_client = up_client_new ();
-  if (up_client_get_lid_is_present (self->priv->up_client))
-    {
-      /* Connect to the "changed" signal to track changes to "lid-is-closed"
-       * property. Connecting to "notify::lid-is-closed" would be preferable,
-       * but currently doesn't work as expected:
-       * https://bugs.freedesktop.org/show_bug.cgi?id=43001
-       */
-
-      g_signal_connect (self->priv->up_client, "changed",
-                        G_CALLBACK (cc_display_panel_up_client_changed), self);
-      cc_display_panel_up_client_changed (self->priv->up_client, self);
-    }
-  else
-    g_clear_object (&self->priv->up_client);
-
   return obj;
 }
