@@ -51,9 +51,7 @@
 #include "network-dialogs.h"
 #include "connection-editor/net-connection-editor.h"
 
-#if HAVE_MM_GLIB
 #include <libmm-glib.h>
-#endif /* HAVE_MM_GLIB */
 
 CC_PANEL_REGISTER (CcNetworkPanel, cc_network_panel)
 
@@ -75,9 +73,7 @@ struct _CcNetworkPanelPrivate
         GtkBuilder       *builder;
         GtkWidget        *treeview;
         NMClient         *client;
-#if HAVE_MM_GLIB
         MMManager        *modem_manager;
-#endif /* HAVE_MM_GLIB */
         NMRemoteSettings *remote_settings;
         gboolean          updating_device;
         guint             nm_warning_idle;
@@ -106,7 +102,7 @@ enum {
 
 enum {
         PROP_0,
-        PROP_ARGV
+        PROP_PARAMETERS
 };
 
 static NetObject *find_in_model_by_id (CcNetworkPanel *panel, const gchar *id, GtkTreeIter *iter_out);
@@ -167,6 +163,24 @@ verify_argv (CcNetworkPanel *self,
 	}
 }
 
+static GPtrArray *
+variant_av_to_string_array (GVariant *array)
+{
+        GVariantIter iter;
+        GVariant *v;
+        GPtrArray *strv;
+        gsize count;
+        count = g_variant_iter_init (&iter, array);
+        strv = g_ptr_array_sized_new (count + 1);
+        while (g_variant_iter_next (&iter, "v", &v)) {
+                if (g_variant_is_of_type (v, G_VARIANT_TYPE_STRING))
+                        g_ptr_array_add (strv, (gpointer *)g_variant_get_string (v, NULL));
+                g_variant_unref (v);
+        }
+        g_ptr_array_add (strv, NULL); /* NULL-terminate the strv data array */
+        return strv;
+}
+
 static void
 cc_network_panel_set_property (GObject      *object,
                                guint         property_id,
@@ -177,14 +191,18 @@ cc_network_panel_set_property (GObject      *object,
         CcNetworkPanelPrivate *priv = self->priv;
 
         switch (property_id) {
-        case PROP_ARGV: {
-                gchar **args;
+        case PROP_PARAMETERS: {
+                GVariant *parameters;
 
                 reset_command_line_args (self);
 
-                args = g_value_get_boxed (value);
+                parameters = g_value_get_variant (value);
+                if (parameters) {
+                        GPtrArray *array;
+                        const gchar **args;
+                        array = variant_av_to_string_array (parameters);
+                        args = array->pdata;
 
-                if (args) {
                         g_debug ("Invoked with operation %s", args[0]);
 
                         if (args[0])
@@ -196,9 +214,10 @@ cc_network_panel_set_property (GObject      *object,
 
                         if (verify_argv (self, (const char **) args) == FALSE) {
                                 reset_command_line_args (self);
+                                g_ptr_array_unref (array);
                                 return;
                         }
-
+                        g_ptr_array_unref (array);
                         g_debug ("Calling handle_argv() after setting property");
                         handle_argv (self);
                 }
@@ -220,9 +239,7 @@ cc_network_panel_dispose (GObject *object)
         g_clear_object (&priv->cancellable);
         g_clear_object (&priv->builder);
         g_clear_object (&priv->client);
-#if HAVE_MM_GLIB
         g_clear_object (&priv->modem_manager);
-#endif /* HAVE_MM_GLIB */
         g_clear_object (&priv->remote_settings);
         g_clear_object (&priv->kill_switch_header);
         g_clear_object (&priv->rfkill);
@@ -383,7 +400,7 @@ cc_network_panel_class_init (CcNetworkPanelClass *klass)
         object_class->finalize = cc_network_panel_finalize;
         object_class->constructed = cc_network_panel_constructed;
 
-        g_object_class_override_property (object_class, PROP_ARGV, "argv");
+        g_object_class_override_property (object_class, PROP_PARAMETERS, "parameters");
 }
 
 static NetObject *
@@ -692,7 +709,6 @@ panel_add_device (CcNetworkPanel *panel, NMDevice *device)
                                    "id", nm_device_get_udi (device),
                                    NULL);
 
-#if HAVE_MM_GLIB
         if (type == NM_DEVICE_TYPE_MODEM &&
             g_str_has_prefix (nm_device_get_udi (device), "/org/freedesktop/ModemManager1/Modem/")) {
                 GDBusObject *modem_object;
@@ -717,7 +733,6 @@ panel_add_device (CcNetworkPanel *panel, NMDevice *device)
                               NULL);
                 g_object_unref (modem_object);
         }
-#endif /* HAVE_MM_GLIB */
 
         /* add as a panel */
         if (device_g_type != NET_TYPE_DEVICE) {
@@ -741,7 +756,6 @@ panel_add_device (CcNetworkPanel *panel, NMDevice *device)
                             PANEL_DEVICES_COLUMN_SORT, panel_device_to_sortable_string (device),
                             PANEL_DEVICES_COLUMN_OBJECT, net_device,
                             -1);
-        g_object_unref (net_device);
         g_signal_connect (device, "state-changed",
                           G_CALLBACK (state_changed_cb), panel);
 
@@ -883,7 +897,6 @@ nm_devices_treeview_clicked_cb (GtkTreeSelection *selection, CcNetworkPanel *pan
                 }
                 i++;
         }
-        g_object_unref (object);
 out:
         g_list_free (panels);
 }
@@ -1309,7 +1322,6 @@ remove_connection (GtkToolButton *button, CcNetworkPanel *panel)
 
         /* delete the object */
         net_object_delete (object);
-        g_object_unref (object);
 }
 
 static void
@@ -1338,9 +1350,7 @@ cc_network_panel_init (CcNetworkPanel *panel)
         GtkTreeSelection *selection;
         GtkWidget *widget;
         GtkWidget *toplevel;
-#if HAVE_MM_GLIB
         GDBusConnection *system_bus;
-#endif /* HAVE_MM_GLIB */
 
         panel->priv = NETWORK_PANEL_PRIVATE (panel);
         g_resources_register (cc_network_get_resource ());
@@ -1390,7 +1400,6 @@ cc_network_panel_init (CcNetworkPanel *panel)
         g_signal_connect (panel->priv->client, "device-removed",
                           G_CALLBACK (device_removed_cb), panel);
 
-#if HAVE_MM_GLIB
         /* Setup ModemManager client */
         system_bus = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, &error);
         if (system_bus == NULL) {
@@ -1409,7 +1418,6 @@ cc_network_panel_init (CcNetworkPanel *panel)
                 }
                 g_object_unref (system_bus);
         }
-#endif /* HAVE_MM_GLIB */
 
         widget = GTK_WIDGET (gtk_builder_get_object (panel->priv->builder,
                                                      "add_toolbutton"));
