@@ -29,7 +29,6 @@
 #include <goabackend/goabackend.h>
 
 #include <egg-list-box/egg-list-box.h>
-#include <libgd/gd-stack.h>
 
 #include "cc-online-accounts-add-account-dialog.h"
 
@@ -45,6 +44,7 @@ struct _GoaPanelAddAccountDialogPrivate
   GError *error;
   GoaClient *client;
   GoaObject *object;
+  GoaProvider *provider;
   GtkListStore *list_store;
   GtkWidget *contacts_grid;
   GtkWidget *mail_grid;
@@ -72,7 +72,8 @@ enum
 G_DEFINE_TYPE (GoaPanelAddAccountDialog, goa_panel_add_account_dialog, GTK_TYPE_DIALOG)
 
 static void
-add_account_dialog_add_account (GoaPanelAddAccountDialog *add_account, GoaProvider *provider)
+add_account_dialog_add_account (GoaPanelAddAccountDialog *add_account,
+                                GoaProvider *provider)
 {
   GoaPanelAddAccountDialogPrivate *priv = add_account->priv;
   GList *children;
@@ -91,6 +92,7 @@ add_account_dialog_add_account (GoaPanelAddAccountDialog *add_account, GoaProvid
     }
   g_list_free (children);
 
+  /* This spins gtk_dialog_run() */
   priv->object = goa_provider_add_account (provider,
                                            priv->client,
                                            GTK_DIALOG (add_account),
@@ -106,12 +108,11 @@ list_box_child_activated_cb (GoaPanelAddAccountDialog *add_account, GtkWidget *c
   provider = g_object_get_data (G_OBJECT (child), "provider");
   if (provider == NULL)
     {
-      gd_stack_set_visible_child_name (GD_STACK (add_account->priv->stack), OTHER_PAGE);
+      gtk_stack_set_visible_child_name (GTK_STACK (add_account->priv->stack), OTHER_PAGE);
       return;
     }
 
   add_account_dialog_add_account (add_account, provider);
-  gtk_dialog_response (GTK_DIALOG (add_account), GTK_RESPONSE_OK);
 }
 
 static void
@@ -187,7 +188,7 @@ add_account_dialog_create_provider_ui (GoaPanelAddAccountDialog *add_account,
     {
       g_object_set_data (G_OBJECT (row_grid), "provider", NULL);
       icon = g_themed_icon_new_with_default_fallbacks ("goa-account");
-      name = g_strdup (_("Other"));
+      name = g_strdup (C_("Online Account", "Other"));
     }
   else
     {
@@ -240,6 +241,7 @@ goa_panel_add_account_dialog_dispose (GObject *object)
 
   g_clear_object (&priv->object);
   g_clear_object (&priv->client);
+  g_clear_object (&priv->provider);
 
   G_OBJECT_CLASS (goa_panel_add_account_dialog_parent_class)->dispose (object);
 }
@@ -250,8 +252,7 @@ goa_panel_add_account_dialog_finalize (GObject *object)
   GoaPanelAddAccountDialog *add_account = GOA_PANEL_ADD_ACCOUNT_DIALOG (object);
   GoaPanelAddAccountDialogPrivate *priv = add_account->priv;
 
-  if (priv->error != NULL)
-    g_error_free (priv->error);
+  g_clear_error (&priv->error);
 
   G_OBJECT_CLASS (goa_panel_add_account_dialog_parent_class)->finalize (object);
 }
@@ -300,8 +301,8 @@ goa_panel_add_account_dialog_init (GoaPanelAddAccountDialog *add_account)
 
   priv->list_store = gtk_list_store_new (N_COLUMNS, GOA_TYPE_PROVIDER, G_TYPE_ICON, G_TYPE_STRING);
 
-  priv->stack = gd_stack_new ();
-  gd_stack_set_transition_type (GD_STACK (priv->stack), GD_STACK_TRANSITION_TYPE_CROSSFADE);
+  priv->stack = gtk_stack_new ();
+  gtk_stack_set_transition_type (GTK_STACK (priv->stack), GTK_STACK_TRANSITION_TYPE_CROSSFADE);
   gtk_container_add (GTK_CONTAINER (grid), priv->stack);
 
   sw = gtk_scrolled_window_new (NULL, NULL);
@@ -309,7 +310,7 @@ goa_panel_add_account_dialog_init (GoaPanelAddAccountDialog *add_account)
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_IN);
   gtk_widget_set_hexpand (sw, TRUE);
   gtk_widget_set_vexpand (sw, TRUE);
-  gd_stack_add_named (GD_STACK (priv->stack), sw, BRANDED_PAGE);
+  gtk_stack_add_named (GTK_STACK (priv->stack), sw, BRANDED_PAGE);
 
   priv->branded_list_box = egg_list_box_new ();
   egg_list_box_add_to_scrolled (priv->branded_list_box, GTK_SCROLLED_WINDOW (sw));
@@ -321,7 +322,7 @@ goa_panel_add_account_dialog_init (GoaPanelAddAccountDialog *add_account)
   grid = gtk_grid_new ();
   gtk_orientable_set_orientation (GTK_ORIENTABLE (grid), GTK_ORIENTATION_VERTICAL);
   gtk_grid_set_row_spacing (GTK_GRID (grid), 12);
-  gd_stack_add_named (GD_STACK (priv->stack), grid, OTHER_PAGE);
+  gtk_stack_add_named (GTK_STACK (priv->stack), grid, OTHER_PAGE);
 
   add_account_dialog_create_group_ui (add_account,
                                       &priv->mail_list_box,
@@ -341,7 +342,7 @@ goa_panel_add_account_dialog_init (GoaPanelAddAccountDialog *add_account)
                                       grid,
                                       _("Resources"));
 
-  gd_stack_set_visible_child_name (GD_STACK (priv->stack), BRANDED_PAGE);
+  gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), BRANDED_PAGE);
 
   gtk_dialog_add_button (GTK_DIALOG (add_account), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
   gtk_dialog_set_default_response (GTK_DIALOG (add_account), GTK_RESPONSE_CANCEL);
@@ -378,6 +379,22 @@ goa_panel_add_account_dialog_new (GoaClient *client)
   return g_object_new (GOA_TYPE_PANEL_ADD_ACCOUNT_DIALOG, "client", client, NULL);
 }
 
+void
+goa_panel_add_account_dialog_set_preseed_data (GoaPanelAddAccountDialog *add_account,
+                                               GoaProvider *provider,
+                                               GVariant *preseed)
+{
+  GoaPanelAddAccountDialogPrivate *priv = add_account->priv;
+
+  g_clear_object (&priv->provider);
+
+  if (provider != NULL)
+    {
+      priv->provider = g_object_ref (provider);
+      goa_provider_set_preseed_data (provider, preseed);
+    }
+}
+
 void
 goa_panel_add_account_dialog_add_provider (GoaPanelAddAccountDialog *add_account, GoaProvider *provider)
 {
@@ -432,6 +449,16 @@ goa_panel_add_account_dialog_add_provider (GoaPanelAddAccountDialog *add_account
     }
 }
 
+void
+goa_panel_add_account_dialog_run (GoaPanelAddAccountDialog *add_account)
+{
+  GoaPanelAddAccountDialogPrivate *priv = add_account->priv;
+  if (priv->provider != NULL)
+    add_account_dialog_add_account (add_account, priv->provider);
+  else
+    gtk_dialog_run (GTK_DIALOG (add_account));
+}
+
 GoaObject *
 goa_panel_add_account_dialog_get_account (GoaPanelAddAccountDialog *add_account, GError **error)
 {
