@@ -84,6 +84,7 @@ struct _CcPowerPanelPrivate
   GtkWidget     *battery_heading;
   GtkWidget     *battery_section;
   GtkWidget     *battery_list;
+  GtkWidget     *battery_capacity;
 
   GtkWidget     *device_heading;
   GtkWidget     *device_section;
@@ -300,6 +301,7 @@ set_primary (CcPowerPanel *panel, UpDevice *device)
   GtkWidget *levelbar;
   gchar *s;
   gdouble energy_full, energy_rate;
+  guint64 capacity;
 
   g_object_get (device,
                 "state", &state,
@@ -314,9 +316,26 @@ set_primary (CcPowerPanel *panel, UpDevice *device)
   else
     time = time_full;
 
+  if (energy_rate > 0)
+    {
+      gchar *time_string, *s;
+      capacity = 3600 * (energy_full / energy_rate);
+      time_string = get_timestring (capacity);
+      s = g_strdup_printf (_("Estimated battery capacity: %s"), time_string);
+      gtk_label_set_label (GTK_LABEL (priv->battery_capacity), s);
+      gtk_widget_show (priv->battery_capacity);
+      g_free (s);
+      g_free (time_string);
+    }
+  else
+    {
+      gtk_widget_hide (priv->battery_capacity);
+    }
+
   details = get_details_string (percentage, state, time);
 
   box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+  gtk_size_group_add_widget (priv->battery_sizegroup, box);
   gtk_widget_set_margin_left (box, 20);
   gtk_widget_set_margin_right (box, 20);
   gtk_widget_set_margin_top (box, 6);
@@ -682,7 +701,7 @@ up_client_changed (UpClient     *client,
                 "power-supply", TRUE,
                 "is-present", TRUE,
                 NULL);
-  for (i = 0; devices != NULL && i < devices->len; i++)
+  for (i = 0; i < devices->len; i++)
     {
       UpDevice *device = (UpDevice*) g_ptr_array_index (devices, i);
       g_object_get (device,
@@ -749,7 +768,7 @@ up_client_changed (UpClient     *client,
   if (!on_ups && n_batteries > 1)
     set_primary (self, composite);
 
-  for (i = 0; devices != NULL && i < devices->len; i++)
+  for (i = 0; i < devices->len; i++)
     {
       UpDevice *device = (UpDevice*) g_ptr_array_index (devices, i);
       g_object_get (device, "kind", &kind, NULL);
@@ -775,7 +794,7 @@ up_client_changed (UpClient     *client,
         }
     }
 
-  g_clear_pointer (&devices, g_ptr_array_unref);
+  g_ptr_array_unref (devices);
   g_object_unref (composite);
 }
 
@@ -1023,7 +1042,7 @@ set_ac_battery_ui_mode (CcPowerPanel *self)
   devices = up_client_get_devices (self->priv->up_client);
   g_debug ("got %d devices from upower\n", devices->len);
 
-  for (i = 0; devices != NULL && i < devices->len; i++)
+  for (i = 0; i < devices->len; i++)
     {
       device = g_ptr_array_index (devices, i);
       g_object_get (device, "kind", &kind, NULL);
@@ -1033,7 +1052,7 @@ set_ac_battery_ui_mode (CcPowerPanel *self)
           break;
         }
     }
-  g_clear_pointer (&devices, g_ptr_array_unref);
+  g_ptr_array_unref (devices);
 
 #ifdef TEST_NO_BATTERIES
   g_print ("forcing no batteries\n");
@@ -1942,6 +1961,11 @@ add_battery_section (CcPowerPanel *self)
   gtk_container_add (GTK_CONTAINER (frame), widget);
   gtk_box_pack_start (GTK_BOX (box), frame, FALSE, TRUE, 0);
 
+  priv->battery_capacity = widget = gtk_label_new ("");
+  gtk_widget_set_margin_top (widget, 6);
+  gtk_widget_set_halign (widget, GTK_ALIGN_CENTER);
+  gtk_style_context_add_class (gtk_widget_get_style_context (widget), GTK_STYLE_CLASS_DIM_LABEL);
+  gtk_box_pack_start (GTK_BOX (box), priv->battery_capacity, FALSE, TRUE, 0);
   gtk_widget_show_all (box);
 }
 
@@ -2084,7 +2108,7 @@ cc_power_panel_init (CcPowerPanel *self)
   gtk_container_add (GTK_CONTAINER (self), box);
   g_object_ref (widget);
   gtk_widget_unparent (widget);
-  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (box), widget);
+  gtk_container_add (GTK_CONTAINER (box), widget);
   g_object_unref (widget);
 
   priv->focus_adjustment = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (box));
