@@ -38,6 +38,9 @@
 #define INPUT_SOURCE_TYPE_XKB "xkb"
 #define INPUT_SOURCE_TYPE_IBUS "ibus"
 
+#define ARROW_NEXT "go-next-symbolic"
+#define ARROW_PREV "go-previous-symbolic"
+
 #define MAIN_WINDOW_WIDTH_RATIO 0.60
 #define FILTER_TIMEOUT 150 /* ms */
 
@@ -139,7 +142,7 @@ padded_label_new (const gchar        *text,
 
   if (direction == ROW_TRAVEL_DIRECTION_BACKWARD)
     {
-      arrow = gtk_image_new_from_icon_name (rtl ? "go-previous-rtl-symbolic" : "go-previous-symbolic",
+      arrow = gtk_image_new_from_icon_name (rtl ? ARROW_NEXT : ARROW_PREV,
                                             GTK_ICON_SIZE_MENU);
       gtk_box_pack_start (GTK_BOX (widget), arrow, FALSE, TRUE, 0);
     }
@@ -153,7 +156,7 @@ padded_label_new (const gchar        *text,
 
   if (direction == ROW_TRAVEL_DIRECTION_FORWARD)
     {
-      arrow = gtk_image_new_from_icon_name (rtl ? "go-next-rtl-symbolic" : "go-next-symbolic",
+      arrow = gtk_image_new_from_icon_name (rtl ? ARROW_PREV : ARROW_NEXT,
                                             GTK_ICON_SIZE_MENU);
       gtk_box_pack_start (GTK_BOX (widget), arrow, FALSE, TRUE, 0);
     }
@@ -221,7 +224,7 @@ input_source_widget_new (GtkWidget   *chooser,
                          const gchar *id)
 {
   CcInputChooserPrivate *priv = GET_PRIVATE (chooser);
-  GtkWidget *widget;
+  GtkWidget *widget = NULL;
 
   if (g_str_equal (type, INPUT_SOURCE_TYPE_XKB))
     {
@@ -389,7 +392,7 @@ static void
 show_locale_widgets (GtkWidget *chooser)
 {
   CcInputChooserPrivate *priv = GET_PRIVATE (chooser);
-  GHashTable *initial;
+  GHashTable *initial = NULL;
   LocaleInfo *info;
   GHashTableIter iter;
 
@@ -1085,7 +1088,7 @@ get_locale_infos (GtkWidget *chooser)
   /* Add a "Other" locale to hold the remaining input sources */
   info = g_new0 (LocaleInfo, 1);
   info->id = g_strdup ("");
-  info->name = g_strdup (_("Other"));
+  info->name = g_strdup (C_("Input Source", "Other"));
   info->unaccented_name = g_strdup ("");
   info->untranslated_name = g_strdup ("");
   g_hash_table_replace (priv->locales, info->id, info);
