@@ -166,15 +166,24 @@ cc_sharing_panel_dispose (GObject *object)
   G_OBJECT_CLASS (cc_sharing_panel_parent_class)->dispose (object);
 }
 
+static const char *
+cc_sharing_panel_get_help_uri (CcPanel *panel)
+{
+  return "help:gnome-help/settings-sharing";
+}
+
 static void
 cc_sharing_panel_class_init (CcSharingPanelClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  CcPanelClass *panel_class = CC_PANEL_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (CcSharingPanelPrivate));
 
   object_class->constructed = cc_sharing_panel_constructed;
   object_class->dispose = cc_sharing_panel_dispose;
+
+  panel_class->get_help_uri = cc_sharing_panel_get_help_uri;
 }
 
 static void
@@ -403,9 +412,6 @@ cc_sharing_panel_add_folder (GtkWidget      *button,
   GtkWidget *dialog;
   GtkListStore *store;
   gchar *folder;
-  GtkTreeIter iter;
-  gboolean valid;
-  gboolean matching;
 
   dialog = gtk_file_chooser_dialog_new (_("Choose a Folder"),
                                         GTK_WINDOW (gtk_widget_get_toplevel (button)),
@@ -422,21 +428,7 @@ cc_sharing_panel_add_folder (GtkWidget      *button,
 
   folder = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dialog));
 
-  for (valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &iter);
-       valid;
-       valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (store), &iter))
-    {
-      gchar *string;
-
-      gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, 0, &string, -1);
-      matching = (g_strcmp0 (string, folder) != 0);
-      g_free (string);
-
-      if (matching)
-        break;
-    }
-
-  if (!matching && folder && !g_str_equal (folder, ""))
+  if (folder && !g_str_equal (folder, ""))
     gtk_list_store_insert_with_values (store, NULL, -1, 0, folder, -1);
 
   g_free (folder);
@@ -750,63 +742,6 @@ cc_sharing_panel_check_schema_available (CcSharingPanel *self,
   return FALSE;
 }
 
-static void
-screen_sharing_show_cb (GtkWidget *widget, CcSharingPanel *self)
-{
-  CcSharingPanelPrivate *priv = self->priv;
-
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("show-password-checkbutton")),
-                                FALSE);
-}
-
-static void
-screen_sharing_hide_cb (GtkWidget *widget, CcSharingPanel *self)
-{
-  GtkSwitch   *pw_switch;
-  GtkEntry    *pw_entry;
-  const gchar *password;
-  CcSharingPanelPrivate *priv = self->priv;
-
-  pw_switch = GTK_SWITCH (WID ("remote-control-require-password-switch"));
-  pw_entry = GTK_ENTRY (WID ("remote-control-password-entry"));
-  password = gtk_entry_get_text (pw_entry);
-
-  if (password == NULL || *password == '\0')
-    gtk_switch_set_active (pw_switch, FALSE);
-}
-
-#define MAX_PASSWORD_SIZE 8
-static void
-screen_sharing_password_insert_text_cb (GtkEditable *editable,
-                                        gchar       *new_text,
-                                        gint         new_text_length,
-                                        gpointer     position,
-                                        gpointer     user_data)
-{
-  int l, available_size;
-
-  l = gtk_entry_buffer_get_bytes (gtk_entry_get_buffer (GTK_ENTRY (editable)));
-
-  if (l + new_text_length <= MAX_PASSWORD_SIZE)
-    return;
-
-  g_signal_stop_emission_by_name (editable, "insert-text");
-  gtk_widget_error_bell (GTK_WIDGET (editable));
-
-  available_size = g_utf8_strlen (new_text, MAX_PASSWORD_SIZE - l);
-  if (available_size == 0)
-    return;
-
-  g_signal_handlers_block_by_func (editable,
-                                   (gpointer) screen_sharing_password_insert_text_cb,
-                                   user_data);
-  gtk_editable_insert_text (editable, new_text, available_size, position);
-  g_signal_handlers_unblock_by_func (editable,
-                                     (gpointer) screen_sharing_password_insert_text_cb,
-                                     user_data);
-}
-#undef MAX_PASSWORD_SIZE
-
 static void
 cc_sharing_panel_setup_screen_sharing_dialog (CcSharingPanel *self)
 {
@@ -817,7 +752,17 @@ cc_sharing_panel_setup_screen_sharing_dialog (CcSharingPanel *self)
                                          WID ("screen-sharing-status-label"));
 
   cc_sharing_panel_bind_switch_to_widgets (WID ("remote-view-switch"),
-                                           WID ("remote-control-box"),
+                                           WID ("remote-control-switch"),
+                                           WID ("remote-control-label"),
+                                           WID ("remote-control-grid"),
+                                           NULL);
+
+  cc_sharing_panel_bind_switch_to_widgets (WID ("remote-control-switch"),
+                                           WID ("approve-all-connections-switch"),
+                                           WID ("remote-control-require-password-switch"),
+                                           WID ("remote-control-require-password-label"),
+                                           WID ("approve-all-connections-label"),
+                                           WID ("password-grid"),
                                            NULL);
 
   cc_sharing_panel_bind_switch_to_widgets (WID ("remote-control-require-password-switch"),
@@ -853,17 +798,6 @@ cc_sharing_panel_setup_screen_sharing_dialog (CcSharingPanel *self)
   g_object_bind_property (WID ("show-password-checkbutton"), "active",
                           WID ("remote-control-password-entry"), "visibility",
                           G_BINDING_SYNC_CREATE);
-
-  /* make sure the password entry is hidden by default */
-  g_signal_connect (priv->screen_sharing_dialog, "show",
-                    G_CALLBACK (screen_sharing_show_cb), self);
-
-  g_signal_connect (priv->screen_sharing_dialog, "hide",
-                    G_CALLBACK (screen_sharing_hide_cb), self);
-
-  /* accept at most 8 bytes in password entry */
-  g_signal_connect (WID ("remote-control-password-entry"), "insert-text",
-                    G_CALLBACK (screen_sharing_password_insert_text_cb), self);
 }
 
 static void
