@@ -165,15 +165,6 @@ complete_dialog (UmAccountDialog *self,
         gtk_widget_hide (GTK_WIDGET (self));
 }
 
-static void
-user_loaded_cb (ActUser         *user,
-                GParamSpec      *pspec,
-                UmAccountDialog *self)
-{
-  finish_action (self);
-  complete_dialog (self, user);
-}
-
 static void
 create_user_done (ActUserManager  *manager,
                   GAsyncResult    *res,
@@ -182,13 +173,14 @@ create_user_done (ActUserManager  *manager,
         ActUser *user;
         GError *error;
 
+        finish_action (self);
+
         /* Note that user is returned without an extra reference */
 
         error = NULL;
         user = act_user_manager_create_user_finish (manager, res, &error);
 
         if (user == NULL) {
-                finish_action (self);
                 g_debug ("Failed to create user: %s", error->message);
                 if (!g_error_matches (error, ACT_USER_MANAGER_ERROR, ACT_USER_MANAGER_ERROR_PERMISSION_DENIED))
                        show_error_dialog (self, _("Failed to add account"), error);
@@ -196,11 +188,7 @@ create_user_done (ActUserManager  *manager,
                 gtk_widget_grab_focus (self->local_name);
         } else {
                 g_debug ("Created user: %s", act_user_get_user_name (user));
-                /* Check if the returned object is fully loaded before returning it */
-                if (act_user_is_loaded (user))
-                        user_loaded_cb (user, NULL, self);
-                else
-                        g_signal_connect (user, "notify::is-loaded", G_CALLBACK (user_loaded_cb), self);
+                complete_dialog (self, user);
         }
 }
 
@@ -274,19 +262,13 @@ on_name_changed (GtkEditable *editable,
         UmAccountDialog *self = UM_ACCOUNT_DIALOG (user_data);
         GtkTreeModel *model;
         const char *name;
-        GtkWidget *entry;
 
         model = gtk_combo_box_get_model (GTK_COMBO_BOX (self->local_username));
         gtk_list_store_clear (GTK_LIST_STORE (model));
 
         name = gtk_entry_get_text (GTK_ENTRY (editable));
-        if (strlen (name) == 0) {
-                entry = gtk_bin_get_child (GTK_BIN (self->local_username));
-                gtk_entry_set_text (GTK_ENTRY (entry), "");
-        } else {
-                generate_username_choices (name, GTK_LIST_STORE (model));
-                gtk_combo_box_set_active (GTK_COMBO_BOX (self->local_username), 0);
-        }
+        generate_username_choices (name, GTK_LIST_STORE (model));
+        gtk_combo_box_set_active (GTK_COMBO_BOX (self->local_username), 0);
 
         dialog_validate (self);
 }
@@ -414,11 +396,6 @@ on_register_user (GObject *source,
         GError *error = NULL;
         ActUser *user;
 
-        if (g_cancellable_is_cancelled (self->cancellable)) {
-                g_object_unref (self);
-                return;
-        }
-
         user = act_user_manager_cache_user_finish (ACT_USER_MANAGER (source), result, &error);
 
         /* This is where we're finally done */
@@ -448,11 +425,6 @@ on_permit_user_login (GObject *source,
         GError *error = NULL;
         gchar *login;
 
-        if (g_cancellable_is_cancelled (self->cancellable)) {
-                g_object_unref (self);
-                return;
-        }
-
         common = UM_REALM_COMMON (source);
         um_realm_common_call_change_login_policy_finish (common, result, &error);
         if (error == NULL) {
@@ -598,11 +570,6 @@ on_join_login (GObject *source,
         GError *error = NULL;
         GBytes *creds;
 
-        if (g_cancellable_is_cancelled (self->cancellable)) {
-                g_object_unref (self);
-                return;
-        }
-
         um_realm_login_finish (result, &creds, &error);
 
         /* Logged in as admin successfully, use creds to join domain */
@@ -650,11 +617,6 @@ on_realm_joined (GObject *source,
         UmAccountDialog *self = UM_ACCOUNT_DIALOG (user_data);
         GError *error = NULL;
 
-        if (g_cancellable_is_cancelled (self->cancellable)) {
-                g_object_unref (self);
-                return;
-        }
-
         um_realm_join_finish (self->selected_realm,
                               result, &error);
 
@@ -689,11 +651,6 @@ on_realm_login (GObject *source,
         GError *error = NULL;
         GBytes *creds = NULL;
 
-        if (g_cancellable_is_cancelled (self->cancellable)) {
-                g_object_unref (self);
-                return;
-        }
-
         um_realm_login_finish (result, &creds, &error);
 
         /*
@@ -775,11 +732,6 @@ on_realm_discover_input (GObject *source,
         GError *error = NULL;
         GList *realms;
 
-        if (g_cancellable_is_cancelled (self->cancellable)) {
-                g_object_unref (self);
-                return;
-        }
-
         realms = um_realm_manager_discover_finish (self->realm_manager,
                                                    result, &error);
 
@@ -830,18 +782,6 @@ enterprise_add_user (UmAccountDialog *self)
         }
 }
 
-static void
-clear_realm_manager (UmAccountDialog *self)
-{
-        if (self->realm_manager) {
-                g_signal_handlers_disconnect_by_func (self->realm_manager,
-                                                      on_manager_realm_added,
-                                                      self);
-                g_object_unref (self->realm_manager);
-                self->realm_manager = NULL;
-        }
-}
-
 static void
 on_realm_manager_created (GObject *source,
                            GAsyncResult *result,
@@ -851,21 +791,15 @@ on_realm_manager_created (GObject *source,
         GError *error = NULL;
         GList *realms, *l;
 
-        clear_realm_manager (self);
+        g_clear_object (&self->realm_manager);
 
         self->realm_manager = um_realm_manager_new_finish (result, &error);
         if (error != NULL) {
                 g_warning ("Couldn't contact realmd service: %s", error->message);
-                g_object_unref (self);
                 g_error_free (error);
                 return;
         }
 
-        if (g_cancellable_is_cancelled (self->cancellable)) {
-                g_object_unref (self);
-                return;
-        }
-
         /* Lookup all the realm objects */
         realms = um_realm_manager_get_realms (self->realm_manager);
         for (l = realms; l != NULL; l = g_list_next (l))
@@ -881,7 +815,6 @@ on_realm_manager_created (GObject *source,
         /* Show the 'Enterprise Login' stuff, and update mode */
         gtk_widget_show (self->enterprise_button);
         mode_change (self, self->mode);
-        g_object_unref (self);
 }
 
 static void
@@ -891,8 +824,7 @@ on_realmd_appeared (GDBusConnection *connection,
                     gpointer user_data)
 {
         UmAccountDialog *self = UM_ACCOUNT_DIALOG (user_data);
-        um_realm_manager_new (self->cancellable, on_realm_manager_created,
-                              g_object_ref (self));
+        um_realm_manager_new (self->cancellable, on_realm_manager_created, self);
 }
 
 static void
@@ -902,7 +834,14 @@ on_realmd_disappeared (GDBusConnection *unused1,
 {
         UmAccountDialog *self = UM_ACCOUNT_DIALOG (user_data);
 
-        clear_realm_manager (self);
+        if (self->realm_manager) {
+                g_signal_handlers_disconnect_by_func (self->realm_manager,
+                                                      on_manager_realm_added,
+                                                      self);
+                g_object_unref (self->realm_manager);
+                self->realm_manager = NULL;
+        }
+
         gtk_list_store_clear (self->enterprise_realms);
         gtk_widget_hide (self->enterprise_button);
         mode_change (self, UM_LOCAL);
