@@ -72,11 +72,6 @@ struct _CcUserPanelPrivate {
         UmPasswordDialog *password_dialog;
         UmPhotoDialog *photo_dialog;
         UmHistoryDialog *history_dialog;
-
-        gint other_accounts;
-        GtkTreeIter *other_iter;
-
-        UmAccountDialog *account_dialog;
 };
 
 static GtkWidget *
@@ -145,14 +140,10 @@ user_added (ActUserManager *um, ActUser *user, CcUserPanelPrivate *d)
         GtkTreeIter iter;
         GtkTreeIter dummy;
         GdkPixbuf *pixbuf;
-        gchar *text, *title;
+        gchar *text;
         GtkTreeSelection *selection;
         gint sort_key;
 
-        if (act_user_is_system_account (user)) {
-                return;
-        }
-
         g_debug ("user added: %d %s\n", act_user_get_uid (user), get_real_or_user_name (user));
         widget = get_widget (d, "list-treeview");
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
@@ -166,7 +157,6 @@ user_added (ActUserManager *um, ActUser *user, CcUserPanelPrivate *d)
                 sort_key = 1;
         }
         else {
-                d->other_accounts++;
                 sort_key = 3;
         }
         gtk_list_store_append (store, &iter);
@@ -187,19 +177,6 @@ user_added (ActUserManager *um, ActUser *user, CcUserPanelPrivate *d)
             !gtk_tree_selection_get_selected (selection, &model, &dummy)) {
                 gtk_tree_selection_select_iter (selection, &iter);
         }
-
-        /* Show heading for other accounts if new one have been added. */
-        if (d->other_accounts == 1 && sort_key == 3) {
-                title = g_strdup_printf ("<small><span foreground=\"#555555\">%s</span></small>", _("Other Accounts"));
-                gtk_list_store_append (store, &iter);
-                gtk_list_store_set (store, &iter,
-                                    TITLE_COL, title,
-                                    HEADING_ROW_COL, TRUE,
-                                    SORT_KEY_COL, 2,
-                                    -1);
-                d->other_iter = gtk_tree_iter_copy (&iter);
-                g_free (title);
-        }
 }
 
 static void
@@ -250,7 +227,6 @@ user_removed (ActUserManager *um, ActUser *user, CcUserPanelPrivate *d)
         GtkListStore *store;
         GtkTreeIter iter, next;
         ActUser *u;
-        gint key;
 
         g_debug ("user removed: %s\n", act_user_get_user_name (user));
         tv = (GtkTreeView *)get_widget (d, "list-treeview");
@@ -259,15 +235,12 @@ user_removed (ActUserManager *um, ActUser *user, CcUserPanelPrivate *d)
         store = GTK_LIST_STORE (model);
         if (gtk_tree_model_get_iter_first (model, &iter)) {
                 do {
-                        gtk_tree_model_get (model, &iter, USER_COL, &u, SORT_KEY_COL, &key, -1);
+                        gtk_tree_model_get (model, &iter, USER_COL, &u, -1);
 
                         if (u != NULL) {
                                 if (act_user_get_uid (user) == act_user_get_uid (u)) {
                                         if (!get_next_user_row (model, &iter, &next))
                                                 get_previous_user_row (model, &iter, &next);
-                                        if (key == 3) {
-                                                d->other_accounts--;
-                                        }
                                         gtk_list_store_remove (store, &iter);
                                         gtk_tree_selection_select_iter (selection, &next);
                                         g_object_unref (u);
@@ -277,13 +250,6 @@ user_removed (ActUserManager *um, ActUser *user, CcUserPanelPrivate *d)
                         }
                 } while (gtk_tree_model_iter_next (model, &iter));
         }
-
-        /* Hide heading for other accounts if last one have been removed. */
-        if (d->other_iter != NULL && d->other_accounts == 0 && key == 3) {
-                gtk_list_store_remove (store, d->other_iter);
-                gtk_tree_iter_free (d->other_iter);
-                d->other_iter = NULL;
-        }
 }
 
 static void show_user (ActUser *user, CcUserPanelPrivate *d);
@@ -351,12 +317,10 @@ select_created_user (GObject *object,
         ActUser *current;
         GtkTreePath *path;
         ActUser *user;
-        uid_t user_uid;
 
         dialog = UM_ACCOUNT_DIALOG (object);
         user = um_account_dialog_finish (dialog, result);
         gtk_widget_destroy (GTK_WIDGET (dialog));
-        d->account_dialog = NULL;
 
         if (user == NULL)
                 return;
@@ -364,22 +328,20 @@ select_created_user (GObject *object,
         tv = (GtkTreeView *)get_widget (d, "list-treeview");
         model = gtk_tree_view_get_model (tv);
         selection = gtk_tree_view_get_selection (tv);
-        user_uid = act_user_get_uid (user);
 
         gtk_tree_model_get_iter_first (model, &iter);
         do {
                 gtk_tree_model_get (model, &iter, USER_COL, &current, -1);
-                if (current) {
-                        if (user_uid == act_user_get_uid (current)) {
-                                path = gtk_tree_model_get_path (model, &iter);
-                                gtk_tree_view_scroll_to_cell (tv, path, NULL, FALSE, 0.0, 0.0);
-                                gtk_tree_selection_select_path (selection, path);
-                                gtk_tree_path_free (path);
-                                g_object_unref (current);
-                                break;
-                        }
+                if (user == current) {
+                        path = gtk_tree_model_get_path (model, &iter);
+                        gtk_tree_view_scroll_to_cell (tv, path, NULL, FALSE, 0.0, 0.0);
+                        gtk_tree_selection_select_path (selection, path);
+                        gtk_tree_path_free (path);
                         g_object_unref (current);
+                        break;
                 }
+                if (current)
+                        g_object_unref (current);
         } while (gtk_tree_model_iter_next (model, &iter));
 
         g_object_unref (user);
@@ -388,8 +350,10 @@ select_created_user (GObject *object,
 static void
 add_user (GtkButton *button, CcUserPanelPrivate *d)
 {
-        d->account_dialog = um_account_dialog_new ();
-        um_account_dialog_show (d->account_dialog, GTK_WINDOW (gtk_widget_get_toplevel (d->main_box)),
+        UmAccountDialog *dialog;
+
+        dialog = um_account_dialog_new ();
+        um_account_dialog_show (dialog, GTK_WINDOW (gtk_widget_get_toplevel (d->main_box)),
                                 d->permission, select_created_user, d);
 }
 
@@ -444,11 +408,6 @@ delete_user_response (GtkWidget         *dialog,
 
         user = get_selected_user (d);
 
-        /* remove autologin */
-        if (act_user_get_automatic_login (user)) {
-                act_user_set_automatic_login (user, FALSE);
-        }
-
         act_user_manager_delete_user_async (d->um,
                                             user,
                                             remove_files,
@@ -636,20 +595,6 @@ get_login_time_text (ActUser *user)
         return text;
 }
 
-static gboolean
-get_autologin_possible (ActUser *user)
-{
-        gboolean locked;
-        gboolean set_password_at_login;
-
-        locked = act_user_get_locked (user);
-        set_password_at_login = (act_user_get_password_mode (user) == ACT_USER_PASSWORD_MODE_SET_AT_LOGIN);
-
-        return !(locked || set_password_at_login);
-}
-
-static void on_permission_changed (GPermission *permission, GParamSpec *pspec, gpointer data);
-
 static void
 show_user (ActUser *user, CcUserPanelPrivate *d)
 {
@@ -690,7 +635,9 @@ show_user (ActUser *user, CcUserPanelPrivate *d)
         g_signal_handlers_block_by_func (widget, autologin_changed, d);
         gtk_switch_set_active (GTK_SWITCH (widget), act_user_get_automatic_login (user));
         g_signal_handlers_unblock_by_func (widget, autologin_changed, d);
-        gtk_widget_set_sensitive (widget, get_autologin_possible (user));
+
+        if (act_user_get_locked (user))
+                gtk_widget_set_sensitive (widget, FALSE);
 
         widget = get_widget (d, "account-language-combo");
         model = um_editable_combo_get_model (UM_EDITABLE_COMBO (widget));
@@ -740,11 +687,10 @@ show_user (ActUser *user, CcUserPanelPrivate *d)
         widget = get_widget (d, "last-login-history-button");
         gtk_widget_set_visible (widget, show);
         gtk_widget_set_sensitive (widget, enable);
-
-        if (d->permission != NULL)
-                on_permission_changed (d->permission, NULL, d);
 }
 
+static void on_permission_changed (GPermission *permission, GParamSpec *pspec, gpointer data);
+
 static void
 selected_user_changed (GtkTreeSelection *selection, CcUserPanelPrivate *d)
 {
@@ -755,6 +701,8 @@ selected_user_changed (GtkTreeSelection *selection, CcUserPanelPrivate *d)
         if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
                 gtk_tree_model_get (model, &iter, USER_COL, &user, -1);
                 show_user (user, d);
+                if (d->permission != NULL)
+                        on_permission_changed (d->permission, NULL, d);
                 gtk_widget_set_sensitive (get_widget (d, "main-user-vbox"), TRUE);
                 g_object_unref (user);
         } else {
@@ -1091,8 +1039,7 @@ on_permission_changed (GPermission *permission,
         }
 
         widget = get_widget (d, "remove-user-toolbutton");
-        gtk_widget_set_sensitive (widget, is_authorized && !self_selected
-                                  && !would_demote_only_admin (user));
+        gtk_widget_set_sensitive (widget, is_authorized && !self_selected);
         if (is_authorized) {
                 setup_tooltip_with_embedded_icon (widget, _("Delete the selected user account"), NULL, NULL);
         }
@@ -1119,23 +1066,14 @@ on_permission_changed (GPermission *permission,
                 remove_unlock_tooltip (get_widget (d, "autologin-switch"));
 
         } else if (is_authorized && act_user_is_local_account (user)) {
-                if (would_demote_only_admin (user)) {
-                        um_editable_combo_set_editable (UM_EDITABLE_COMBO (get_widget (d, "account-type-combo")), FALSE);
-                } else {
-                        um_editable_combo_set_editable (UM_EDITABLE_COMBO (get_widget (d, "account-type-combo")), TRUE);
-                }
+                um_editable_combo_set_editable (UM_EDITABLE_COMBO (get_widget (d, "account-type-combo")), TRUE);
                 remove_unlock_tooltip (get_widget (d, "account-type-combo"));
-
-                gtk_widget_set_sensitive (GTK_WIDGET (get_widget (d, "autologin-switch")), get_autologin_possible (user));
+                gtk_widget_set_sensitive (GTK_WIDGET (get_widget (d, "autologin-switch")), TRUE);
                 remove_unlock_tooltip (get_widget (d, "autologin-switch"));
         }
         else {
                 um_editable_combo_set_editable (UM_EDITABLE_COMBO (get_widget (d, "account-type-combo")), FALSE);
-                if (would_demote_only_admin (user)) {
-                        remove_unlock_tooltip (get_widget (d, "account-type-combo"));
-                } else {
-                        add_unlock_tooltip (get_widget (d, "account-type-combo"));
-                }
+                add_unlock_tooltip (get_widget (d, "account-type-combo"));
                 gtk_widget_set_sensitive (GTK_WIDGET (get_widget (d, "autologin-switch")), FALSE);
                 add_unlock_tooltip (get_widget (d, "autologin-switch"));
         }
@@ -1298,8 +1236,14 @@ setup_main_window (CcUserPanelPrivate *d)
                             -1);
         g_free (title);
 
-        d->other_accounts = 0;
-        d->other_iter = NULL;
+        title = g_strdup_printf ("<small><span foreground=\"#555555\">%s</span></small>", _("Other Accounts"));
+        gtk_list_store_append (store, &iter);
+        gtk_list_store_set (store, &iter,
+                            TITLE_COL, title,
+                            HEADING_ROW_COL, TRUE,
+                            SORT_KEY_COL, 2,
+                            -1);
+        g_free (title);
 
         column = gtk_tree_view_column_new ();
         cell = gtk_cell_renderer_pixbuf_new ();
@@ -1441,10 +1385,6 @@ cc_user_panel_dispose (GObject *object)
 {
         CcUserPanelPrivate *priv = UM_USER_PANEL (object)->priv;
 
-        if (priv->um) {
-                g_signal_handlers_disconnect_by_data (priv->um, priv);
-                priv->um = NULL;
-        }
         if (priv->builder) {
                 g_object_unref (priv->builder);
                 priv->builder = NULL;
@@ -1461,10 +1401,6 @@ cc_user_panel_dispose (GObject *object)
                 um_history_dialog_free (priv->history_dialog);
                 priv->history_dialog = NULL;
         }
-        if (priv->account_dialog) {
-                gtk_dialog_response (GTK_DIALOG (priv->account_dialog), GTK_RESPONSE_DELETE_EVENT);
-                priv->account_dialog = NULL;
-        }
         if (priv->language_chooser) {
                 gtk_widget_destroy (priv->language_chooser);
                 priv->language_chooser = NULL;
@@ -1473,10 +1409,6 @@ cc_user_panel_dispose (GObject *object)
                 g_object_unref (priv->permission);
                 priv->permission = NULL;
         }
-        if (priv->other_iter) {
-                gtk_tree_iter_free (priv->other_iter);
-                priv->other_iter = NULL;
-        }
         G_OBJECT_CLASS (cc_user_panel_parent_class)->dispose (object);
 }
 
