@@ -94,7 +94,7 @@ icon_shape_renderer (cairo_t        *cr,
                                                        (gint)height,
                                                        GTK_ICON_LOOKUP_FORCE_SIZE | GTK_ICON_LOOKUP_USE_BUILTIN);
                 pixbuf = gtk_icon_info_load_icon (info, NULL);
-                gtk_icon_info_free (info);
+                g_object_unref (info);
 
                 cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
                 cairo_reset_clip (cr);
@@ -559,7 +559,7 @@ void
 generate_username_choices (const gchar  *name,
                            GtkListStore *store)
 {
-        gboolean in_use;
+        gboolean in_use, same_as_initial;
         char *lc_name, *ascii_name, *stripped_name;
         char **words1;
         char **words2 = NULL;
@@ -707,7 +707,8 @@ generate_username_choices (const gchar  *name,
         }
 
         in_use = is_username_used (item1->str);
-        if (nwords2 > 0 && !in_use && !g_ascii_isdigit (item1->str[0])) {
+        same_as_initial = (g_strcmp0 (item0->str, item1->str) == 0);
+        if (!same_as_initial && nwords2 > 0 && !in_use && !g_ascii_isdigit (item1->str[0])) {
                 gtk_list_store_append (store, &iter);
                 gtk_list_store_set (store, &iter, 0, item1->str, -1);
                 g_hash_table_insert (items, item1->str, item1->str);
@@ -1033,42 +1034,3 @@ set_user_icon_data (ActUser   *user,
 
         g_free (path);
 }
-
-static guint
-get_num_active_admin (ActUserManager *um)
-{
-        GSList *list;
-        GSList *l;
-        guint num_admin = 0;
-
-        list = act_user_manager_list_users (um);
-        for (l = list; l != NULL; l = l->next) {
-                ActUser *u = l->data;
-                if (act_user_get_account_type (u) == ACT_USER_ACCOUNT_TYPE_ADMINISTRATOR && !act_user_get_locked (u)) {
-                        num_admin++;
-                }
-        }
-        g_slist_free (list);
-
-        return num_admin;
-}
-
-gboolean
-would_demote_only_admin (ActUser *user)
-{
-
-        ActUserManager *um = act_user_manager_get_default ();
-
-        /* Prevent the user from demoting the only admin account.
-         * Returns TRUE when user is an administrator and there is only
-         * one enabled administrator. */
-
-        if (act_user_get_account_type (user) == ACT_USER_ACCOUNT_TYPE_STANDARD
-            ||  act_user_get_locked (user))
-                return FALSE;
-
-        if (get_num_active_admin (um) > 1)
-                return FALSE;
-
-        return TRUE;
-}
