@@ -22,6 +22,10 @@
 
 #include <config.h>
 
+#ifdef FAKE_AREA
+#include <gdk/gdk.h>
+#endif /* FAKE_AREA */
+
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 
@@ -30,7 +34,7 @@
 #include "cc-wacom-mapping-panel.h"
 #include "cc-wacom-stylus-page.h"
 #include "gsd-enums.h"
-#include "gui_gtk.h"
+#include "calibrator-gui.h"
 
 #include <string.h>
 
@@ -46,9 +50,10 @@ G_DEFINE_TYPE (CcWacomPage, cc_wacom_page, GTK_TYPE_BOX)
 #define THRESHOLD_MISCLICK	15
 #define THRESHOLD_DOUBLECLICK	7
 
-#define ACTION_TYPE_KEY         "action-type"
-#define CUSTOM_ACTION_KEY       "custom-action"
+#define ACTION_TYPE_KEY            "action-type"
+#define CUSTOM_ACTION_KEY          "custom-action"
 #define KEY_CUSTOM_ELEVATOR_ACTION "custom-elevator-action"
+#define OLED_LABEL                 "oled-label"
 
 enum {
 	MAPPING_DESCRIPTION_COLUMN,
@@ -143,15 +148,19 @@ get_layout_type (GsdWacomDevice *device)
 }
 
 static void
-set_calibration (gint      *cal,
-                 gsize      ncal,
-                 GSettings *settings)
+set_calibration (GsdWacomDevice *device,
+                 const gint      display_width,
+                 const gint      display_height,
+                 gint           *cal,
+                 gsize           ncal,
+                 GSettings      *settings)
 {
 	GVariant    *current; /* current calibration */
 	GVariant    *array;   /* new calibration */
+	GVariant    *last_resolution;
 	GVariant   **tmp;
 	gsize        nvalues;
-	int          i;
+	gint         i;
 
 	current = g_settings_get_value (settings, "area");
 	g_variant_get_fixed_array (current, &nvalues, sizeof (gint32));
@@ -168,6 +177,13 @@ set_calibration (gint      *cal,
 	g_settings_set_value (settings, "area", array);
 
 	g_free (tmp);
+
+	/* set the last-calibration-resolution */
+	last_resolution = g_variant_new ("(ii)", display_width, display_height);
+	settings = gsd_wacom_device_get_settings (device);
+	g_settings_set_value (settings,
+			      "last-calibrated-resolution",
+			      last_resolution);
 }
 
 static void
@@ -178,7 +194,7 @@ finish_calibration (CalibArea *area,
 	CcWacomPagePrivate *priv = page->priv;
 	XYinfo axis;
 	gboolean swap_xy;
-	int cal[4];
+	gint cal[4], display_width, display_height;
 
 	if (calib_area_finish (area, &axis, &swap_xy)) {
 		cal[0] = axis.x_min;
@@ -186,7 +202,12 @@ finish_calibration (CalibArea *area,
 		cal[2] = axis.x_max;
 		cal[3] = axis.y_max;
 
-		set_calibration(cal, 4, priv->wacom_settings);
+		calib_area_get_display_size (area, &display_width, &display_height);
+
+		set_calibration (page->priv->stylus,
+				 display_width,
+				 display_height,
+				 cal, 4, priv->wacom_settings);
 	}
 
 	calib_area_free (area);
@@ -267,7 +288,26 @@ calibrate_button_clicked_cb (GtkButton   *button,
 	    calibration[2] == -1 &&
 	    calibration[3] == -1) {
 		gint *device_cal;
+
+#ifdef FAKE_AREA
+		GdkScreen *screen;
+		screen = gdk_screen_get_default ();
+
+		device_cal = g_new0 (int, 4);
+		device_cal[0] = 0;
+		device_cal[1] = gdk_screen_get_width (screen);
+		device_cal[2] = 0;
+		device_cal[3] = gdk_screen_get_height (screen);
+#else
 		device_cal = gsd_wacom_device_get_area (page->priv->stylus);
+
+		if (device_cal == NULL) {
+			g_warning ("Failed to get device's area. "
+				   "Not running calibration.");
+			return;
+		}
+#endif /* FAKE_AREA */
+
 		for (i = 0; i < 4; i++)
 			calibration[i] = device_cal[i];
 		g_free (device_cal);
@@ -537,6 +577,9 @@ accel_edited_callback (GtkCellRendererText   *cell,
 	    g_strfreev (strv);
   } else {
     g_settings_set_string (button->settings, CUSTOM_ACTION_KEY, str);
+    g_free (str);
+    str = gtk_accelerator_get_label (keyval, mask);
+    g_settings_set_string (button->settings, OLED_LABEL, str);
   }
   g_settings_set_enum (button->settings, ACTION_TYPE_KEY, GSD_WACOM_ACTION_TYPE_CUSTOM);
   g_free (str);
@@ -598,6 +641,7 @@ accel_cleared_callback (GtkCellRendererText *cell,
   } else {
 	  g_settings_set_enum (button->settings, ACTION_TYPE_KEY, GSD_WACOM_ACTION_TYPE_NONE);
 	  g_settings_set_string (button->settings, CUSTOM_ACTION_KEY, "");
+	  g_settings_set_string (button->settings, OLED_LABEL, "");
   }
 }
 
@@ -1033,6 +1077,14 @@ display_clicked_cb (GtkButton   *button,
 	return TRUE;
 }
 
+static gboolean
+mouse_clicked_cb (GtkButton   *button,
+		  CcWacomPage *page)
+{
+	cc_wacom_panel_switch_to_panel (page->priv->panel, "mouse");
+	return TRUE;
+}
+
 /* Boilerplate code goes below */
 
 static void
@@ -1158,6 +1210,9 @@ cc_wacom_page_init (CcWacomPage *self)
 	g_signal_connect (G_OBJECT (WID ("display-link")), "activate-link",
 			  G_CALLBACK (display_clicked_cb), self);
 
+	g_signal_connect (G_OBJECT (WID ("mouse-link")), "activate-link",
+			  G_CALLBACK (mouse_clicked_cb), self);
+
 	g_signal_connect (G_OBJECT (WID ("display-mapping-button")), "clicked",
 			  G_CALLBACK (display_mapping_button_clicked_cb), self);
 
@@ -1271,14 +1326,28 @@ remove_display_link (CcWacomPagePrivate *priv)
                                  "top_attach", 2, NULL);
 }
 
+static void
+remove_mouse_link (CcWacomPagePrivate *priv)
+{
+        gtk_widget_destroy (WID ("mouse-link"));
+
+        gtk_container_child_set (CWID ("main-grid"),
+                                 WID ("tablet-buttons-box"),
+                                 "top_attach", 2, NULL);
+}
+
 static void
 update_tablet_ui (CcWacomPage *page,
 		  int          layout)
 {
 	CcWacomPagePrivate *priv;
 	gboolean has_monitor = FALSE;
+	GsdWacomStylus *puck;
 
 	priv = page->priv;
+	puck = gsd_wacom_device_get_stylus_for_type (priv->stylus, WACOM_STYLUS_TYPE_PUCK);
+	if (puck == NULL)
+		remove_mouse_link (priv);
 
 	/* Hide the pad buttons if no pad is present */
 	gtk_widget_set_visible (WID ("map-buttons-button"), priv->pad != NULL);
@@ -1302,7 +1371,6 @@ update_tablet_ui (CcWacomPage *page,
 		if (gsd_wacom_device_get_display_monitor (priv->stylus) >= 0)
 			has_monitor = TRUE;
 		gtk_widget_set_sensitive (WID ("button-calibrate"), has_monitor);
-		gtk_widget_show (WID ("display-link"));
 
 		gtk_container_child_set (CWID ("main-grid"),
 					 WID ("tablet-buttons-box"),
