@@ -30,10 +30,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <gdk/gdkx.h>
 #include <string.h>
-#include <libgd/gd-stack.h>
-#include <libgd/gd-header-bar.h>
-#include <libgd/gd-header-button.h>
-#include <libgd/gd-styled-text-renderer.h>
+#include <libgd/gd.h>
 
 #include "cc-panel.h"
 #include "cc-shell.h"
@@ -59,6 +56,8 @@ G_DEFINE_TYPE_WITH_CODE (CcWindow, cc_window, GTK_TYPE_APPLICATION_WINDOW,
 
 #define MIN_ICON_VIEW_HEIGHT 300
 
+#define MOUSE_BACK_BUTTON 8
+
 #define DEFAULT_WINDOW_TITLE N_("Settings")
 #define DEFAULT_WINDOW_ICON_NAME "preferences-desktop"
 
@@ -109,11 +108,9 @@ enum
 
 static gboolean cc_window_set_active_panel_from_id (CcShell      *shell,
                                                     const gchar  *start_id,
-                                                    const gchar **argv,
+                                                    GVariant     *parameters,
                                                     GError      **err);
 
-static gint get_monitor_height (CcWindow *self);
-
 static const gchar *
 get_icon_name_from_g_icon (GIcon *gicon)
 {
@@ -139,7 +136,7 @@ get_icon_name_from_g_icon (GIcon *gicon)
 static gboolean
 activate_panel (CcWindow           *self,
                 const gchar        *id,
-                const gchar       **argv,
+                GVariant           *parameters,
                 const gchar        *name,
                 GIcon              *gicon)
 {
@@ -150,7 +147,7 @@ activate_panel (CcWindow           *self,
   if (!id)
     return FALSE;
 
-  priv->current_panel = GTK_WIDGET (cc_panel_loader_load_by_name (CC_SHELL (self), id, argv));
+  priv->current_panel = GTK_WIDGET (cc_panel_loader_load_by_name (CC_SHELL (self), id, parameters));
   cc_shell_set_active_panel (CC_SHELL (self), CC_PANEL (priv->current_panel));
   gtk_widget_show (priv->current_panel);
 
@@ -161,17 +158,17 @@ activate_panel (CcWindow           *self,
 
   gtk_container_add (GTK_CONTAINER (box), priv->current_panel);
 
-  gd_stack_add_named (GD_STACK (priv->stack), box, id);
+  gtk_stack_add_named (GTK_STACK (priv->stack), box, id);
 
   /* switch to the new panel */
   gtk_widget_show (box);
-  gd_stack_set_visible_child_name (GD_STACK (priv->stack), id);
+  gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), id);
 
   /* set the title of the window */
   icon_name = get_icon_name_from_g_icon (gicon);
 
   gtk_window_set_role (GTK_WINDOW (self), id);
-  gd_header_bar_set_title (GD_HEADER_BAR (priv->header), name);
+  gtk_header_bar_set_title (GTK_HEADER_BAR (priv->header), name);
   gtk_window_set_default_icon_name (icon_name);
   gtk_window_set_icon_name (GTK_WINDOW (self), icon_name);
 
@@ -218,7 +215,7 @@ shell_show_overview_page (CcWindow *self)
 {
   CcWindowPrivate *priv = self->priv;
 
-  gd_stack_set_visible_child_name (GD_STACK (priv->stack), OVERVIEW_PAGE);
+  gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), OVERVIEW_PAGE);
 
   if (priv->current_panel_box)
     gtk_container_remove (GTK_CONTAINER (priv->stack), priv->current_panel_box);
@@ -240,7 +237,7 @@ shell_show_overview_page (CcWindow *self)
 
   /* reset window title and icon */
   gtk_window_set_role (GTK_WINDOW (self), NULL);
-  gd_header_bar_set_title (GD_HEADER_BAR (priv->header), NULL);
+  gtk_header_bar_set_title (GTK_HEADER_BAR (priv->header), NULL);
   gtk_window_set_default_icon_name (DEFAULT_WINDOW_ICON_NAME);
   gtk_window_set_icon_name (GTK_WINDOW (self), DEFAULT_WINDOW_ICON_NAME);
 
@@ -621,7 +618,7 @@ search_entry_changed_cb (GtkEntry *entry,
   else
     {
       gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (priv->search_filter));
-      gd_stack_set_visible_child_name (GD_STACK (priv->stack), SEARCH_PAGE);
+      gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), SEARCH_PAGE);
     }
 }
 
@@ -632,8 +629,7 @@ search_entry_key_press_event_cb (GtkEntry        *entry,
 {
   CcWindowPrivate *priv = self->priv;
 
-  if (event->keyval == GDK_KEY_Return &&
-      g_strcmp0 (priv->filter_string, "") != 0)
+  if (event->keyval == GDK_KEY_Return)
     {
       GtkTreePath *path;
       GtkTreeSelection *selection;
@@ -883,7 +879,7 @@ previous_button_clicked_cb (GtkButton *button,
 }
 
 static void
-stack_page_notify_cb (GdStack     *stack,
+stack_page_notify_cb (GtkStack     *stack,
                       GParamSpec  *spec,
                       CcWindow    *self)
 {
@@ -891,30 +887,18 @@ stack_page_notify_cb (GdStack     *stack,
   int nat_height;
   const char *id;
 
-  id = gd_stack_get_visible_child_name (stack);
+  id = gtk_stack_get_visible_child_name (stack);
 
   /* make sure the home button is shown on all pages except the overview page */
 
   if (g_strcmp0 (id, OVERVIEW_PAGE) == 0 || g_strcmp0 (id, SEARCH_PAGE) == 0)
     {
-      gint header_height, maximum_height;
-
       gtk_widget_hide (priv->previous_button);
       gtk_widget_show (priv->search_entry);
       gtk_widget_hide (priv->lock_button);
 
       gtk_widget_get_preferred_height_for_width (GTK_WIDGET (priv->main_vbox),
                                                  FIXED_WIDTH, NULL, &nat_height);
-      gtk_widget_get_preferred_height_for_width (GTK_WIDGET (priv->header),
-                                                 FIXED_WIDTH, NULL, &header_height);
-
-      /* find the maximum height by using the monitor height minus an allowance
-       * for title bar, etc. */
-      maximum_height = get_monitor_height (self) - 100;
-
-      if (maximum_height > 0 && nat_height + header_height > maximum_height)
-        nat_height = maximum_height - header_height;
-
       gtk_scrolled_window_set_min_content_height (GTK_SCROLLED_WINDOW (priv->scrolled_window),
                                                   priv->small_screen == SMALL_SCREEN_TRUE ? SMALL_SCREEN_FIXED_HEIGHT : nat_height);
     }
@@ -949,7 +933,7 @@ _shell_embed_widget_in_header (CcShell      *shell,
 static gboolean
 cc_window_set_active_panel_from_id (CcShell      *shell,
                                     const gchar  *start_id,
-                                    const gchar **argv,
+                                    GVariant     *parameters,
                                     GError      **err)
 {
   GtkTreeIter iter;
@@ -959,10 +943,10 @@ cc_window_set_active_panel_from_id (CcShell      *shell,
   CcWindowPrivate *priv = CC_WINDOW (shell)->priv;
   GtkWidget *old_panel;
 
-  /* When loading the same panel again, just set the argv */
+  /* When loading the same panel again, just set its parameters */
   if (g_strcmp0 (priv->current_panel_id, start_id) == 0)
     {
-      g_object_set (G_OBJECT (priv->current_panel), "argv", argv, NULL);
+      g_object_set (G_OBJECT (priv->current_panel), "parameters", parameters, NULL);
       return TRUE;
     }
 
@@ -1010,7 +994,7 @@ cc_window_set_active_panel_from_id (CcShell      *shell,
     {
       g_warning ("Could not find settings panel \"%s\"", start_id);
     }
-  else if (activate_panel (CC_WINDOW (shell), start_id, argv,
+  else if (activate_panel (CC_WINDOW (shell), start_id, parameters,
                            name, gicon) == FALSE)
     {
       /* Failed to activate the panel for some reason,
@@ -1036,11 +1020,11 @@ cc_window_set_active_panel_from_id (CcShell      *shell,
 static gboolean
 _shell_set_active_panel_from_id (CcShell      *shell,
                                  const gchar  *start_id,
-                                 const gchar **argv,
+                                 GVariant     *parameters,
                                  GError      **err)
 {
   add_current_panel_to_history (shell, start_id);
-  return cc_window_set_active_panel_from_id (shell, start_id, argv, err);
+  return cc_window_set_active_panel_from_id (shell, start_id, parameters, err);
 }
 
 static GtkWidget *
@@ -1085,10 +1069,6 @@ set_active_panel (CcWindow *shell,
         {
           shell->priv->active_panel = g_object_ref (panel);
         }
-      else
-        {
-          shell_show_overview_page (shell);
-        }
       g_object_notify (G_OBJECT (shell), "active-panel");
     }
 }
@@ -1179,6 +1159,17 @@ cc_window_class_init (CcWindowClass *klass)
   g_object_class_override_property (object_class, PROP_ACTIVE_PANEL, "active-panel");
 }
 
+static gboolean
+window_button_release_event (GtkWidget          *win,
+			     GdkEventButton     *event,
+			     CcWindow           *self)
+{
+  /* back button */
+  if (event->button == MOUSE_BACK_BUTTON)
+    shell_show_overview_page (self);
+  return FALSE;
+}
+
 static gboolean
 window_key_press_event (GtkWidget   *win,
                         GdkEventKey *event,
@@ -1220,7 +1211,7 @@ window_key_press_event (GtkWidget   *win,
             break;
           case GDK_KEY_W:
           case GDK_KEY_w:
-            if (g_strcmp0 (gd_stack_get_visible_child_name (GD_STACK (self->priv->stack)), OVERVIEW_PAGE) != 0)
+            if (g_strcmp0 (gtk_stack_get_visible_child_name (GTK_STACK (self->priv->stack)), OVERVIEW_PAGE) != 0)
               shell_show_overview_page (self);
             retval = TRUE;
             break;
@@ -1228,7 +1219,7 @@ window_key_press_event (GtkWidget   *win,
     }
   else if (state == GDK_MOD1_MASK && event->keyval == GDK_KEY_Up)
     {
-      if (g_strcmp0 (gd_stack_get_visible_child_name (GD_STACK (self->priv->stack)), OVERVIEW_PAGE) != 0)
+      if (g_strcmp0 (gtk_stack_get_visible_child_name (GTK_STACK (self->priv->stack)), OVERVIEW_PAGE) != 0)
         shell_show_overview_page (self);
       retval = TRUE;
     }
@@ -1248,9 +1239,6 @@ get_monitor_height (CcWindow *self)
   GdkScreen *screen;
   GdkRectangle rect;
 
-  if (self->priv->monitor_num < 0)
-    return 0;
-
   /* We cannot use workarea here, as this wouldn't
    * be updated when we read it after a monitors-changed signal */
   screen = gtk_widget_get_screen (GTK_WIDGET (self));
@@ -1316,7 +1304,7 @@ update_small_screen_settings (CcWindow *self)
   self->priv->small_screen = small;
 
   /* And update the minimum sizes */
-  stack_page_notify_cb (GD_STACK (self->priv->stack), NULL, self);
+  stack_page_notify_cb (GTK_STACK (self->priv->stack), NULL, self);
 }
 
 static gboolean
@@ -1385,7 +1373,7 @@ create_main_page (CcWindow *self)
   gtk_style_context_add_class (context, "view");
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (priv->scrolled_window),
                                   GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-  gd_stack_add_named (GD_STACK (priv->stack), priv->scrolled_window, OVERVIEW_PAGE);
+  gtk_stack_add_named (GTK_STACK (priv->stack), priv->scrolled_window, OVERVIEW_PAGE);
 
   priv->main_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
   gtk_widget_set_margin_top (priv->main_vbox, 8);
@@ -1410,7 +1398,7 @@ create_search_page (CcWindow *self)
   priv->search_scrolled = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (priv->search_scrolled),
                                   GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-  gd_stack_add_named (GD_STACK (priv->stack), priv->search_scrolled, SEARCH_PAGE);
+  gtk_stack_add_named (GTK_STACK (priv->stack), priv->search_scrolled, SEARCH_PAGE);
 
   /* setup search functionality */
   setup_search (self);
@@ -1422,23 +1410,20 @@ create_header (CcWindow *self)
   CcWindowPrivate *priv = self->priv;
   GtkWidget *button;
   AtkObject *accessible;
-  gboolean rtl;
 
-  rtl = (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL);
-
-  priv->header = gd_header_bar_new ();
+  priv->header = gtk_header_bar_new ();
 
   priv->previous_button = button = gd_header_simple_button_new ();
   gd_header_button_set_symbolic_icon_name (GD_HEADER_BUTTON (button),
-                                           rtl ? "go-previous-rtl-symbolic" : "go-previous-symbolic");
+                                           "go-previous-symbolic");
   gtk_widget_set_no_show_all (button, TRUE);
   accessible = gtk_widget_get_accessible (button);
   atk_object_set_name (accessible, _("All Settings"));
-  gd_header_bar_pack_start (GD_HEADER_BAR (priv->header), button);
+  gtk_header_bar_pack_start (GTK_HEADER_BAR (priv->header), button);
   g_signal_connect (button, "clicked", G_CALLBACK (previous_button_clicked_cb), self);
 
   priv->top_right_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-  gd_header_bar_pack_end (GD_HEADER_BAR (priv->header), priv->top_right_box);
+  gtk_header_bar_pack_end (GTK_HEADER_BAR (priv->header), priv->top_right_box);
 
   priv->search_entry = gtk_search_entry_new ();
   gtk_container_add (GTK_CONTAINER (priv->top_right_box), priv->search_entry);
@@ -1465,9 +1450,9 @@ create_window (CcWindow *self)
   create_header (self);
   gtk_box_pack_start (GTK_BOX (box), priv->header, FALSE, FALSE, 0);
 
-  priv->stack = gd_stack_new ();
-  gd_stack_set_homogeneous (GD_STACK (priv->stack), TRUE);
-  gd_stack_set_transition_type (GD_STACK (priv->stack), GD_STACK_TRANSITION_TYPE_CROSSFADE);
+  priv->stack = gtk_stack_new ();
+  gtk_stack_set_homogeneous (GTK_STACK (priv->stack), TRUE);
+  gtk_stack_set_transition_type (GTK_STACK (priv->stack), GTK_STACK_TRANSITION_TYPE_CROSSFADE);
   gtk_box_pack_start (GTK_BOX (box), priv->stack, TRUE, TRUE, 0);
 
   create_main_page (self);
@@ -1481,6 +1466,10 @@ create_window (CcWindow *self)
   g_signal_connect (self, "notify::application", G_CALLBACK (application_set_cb), self);
   g_signal_connect_after (self, "key_press_event",
                           G_CALLBACK (window_key_press_event), self);
+  gtk_widget_add_events (GTK_WIDGET (self), GDK_BUTTON_RELEASE_MASK);
+  g_signal_connect (self, "button-release-event",
+                    G_CALLBACK (window_button_release_event), self);
+
   g_signal_connect (self, "notify::window", G_CALLBACK (gdk_window_set_cb), self);
 
   g_signal_connect (priv->stack, "notify::visible-child",
@@ -1506,7 +1495,7 @@ cc_window_init (CcWindow *self)
   /* keep a list of custom widgets to unload on panel change */
   priv->custom_widgets = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 
-  stack_page_notify_cb (GD_STACK (priv->stack), NULL, self);
+  stack_page_notify_cb (GTK_STACK (priv->stack), NULL, self);
 }
 
 CcWindow *
@@ -1533,5 +1522,5 @@ cc_window_present (CcWindow *center)
 void
 cc_window_show (CcWindow *center)
 {
-  gtk_window_present (GTK_WINDOW (center));
+  gtk_widget_show (GTK_WIDGET (center));
 }
